//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//                     (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	gtimerge.cc: merge a list of GTIs into one
//
//	DESCRIPTION:
//		The task takes a list of input GTI tables, merges them (logical
//		AND or OR) and writes the resulting new GTI table to a new data
//		set.
//
//	CHANGE HISTORY:
//		UL, 2000-02-16: 1.0.0: - initial version
//		UL, 2000-05-25: 1.1.0: - new parameters 'withgtitable'/'gtitable'
//		UL, 2001-05-30: 1.2.0: - add warning message in case of empty
//								 result GTI table (as described in doc)
//		UL, 2003-01-20: 1.3.0: - added 'plotmergeresult' parameter to plot
//								 result of merging process
//
//	AUTHOR:
//		U. Lammers, Wed Feb 16 17:27:23 MET 2000
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al., 
//
//	$Header: /home/sasbuild/repositories/sasdev/task_devel/packages/gtimerge/src/gtimerge.cc,v 1.5 2003/01/20 14:22:15 ulammers Exp $

#include "param.h"
#include "stringutil.h"
#include "OGIPgti.h"

#include "dsplot.h"
#include "gtimerge.h"

void gtimerge()
{
	vector<string> inTableNames(stringListParameter("tables"));
	if (!inTableNames.size())
		errstr<<"Need at least one GTI table"<<Sas::error("NoGtiTables");

	GtiList mergedList;
	{
		InOGIPgtiTable gtiTab(inTableNames[0]);
		gtiTab>>mergedList;
	}

	string mergeMode(uppercase(stringParameter("mergemode")));
	bool orMode = mergeMode == "OR";
	for (unsigned n=1; n<inTableNames.size(); ++n) {
		InOGIPgtiTable inTab(inTableNames[n]);
		GtiList gl;
		inTab>>gl;

		if (orMode)
			mergedList |= gl;
		else
			mergedList &= gl;
	}

	bool withgtitable(booleanParameter("withgtitable"));
	string outTableName = withgtitable ? stringParameter("gtitable")
									   : inTableNames[0];
	OutOGIPgtiTable outTab(outTableName);

	if (mergedList.empty())
		errstr<<"The list of GTIs resulting from the merging of all input "
				"GTIs is empty. Filtering with this empty list will yield "
				"a likewise empty output table."
			  <<Sas::warning("EmptyOutputGTItable");

	outTab<<mergedList;

	if (booleanParameter("plotmergeresult")) {
		Info::Task *o = task("dsplot");
		o->setParameter("withtable", false);
		o->setParameter("title", string("Result of GTI merging (") +
						mergeMode + ")");
		o->setParameter("withoffsetx", true);
		inTableNames.insert(inTableNames.begin(), outTableName);
		o->setParameter("withxranges", true);
		o->setParameter("xrangetables", inTableNames);
		o->setParameter("xrangefromcol", GTISTARTCOLNAME);
		o->setParameter("xrangetocol", GTISTOPCOLNAME);
		o->exec();
	}
} // gtimerge()
